(ns com.aurellem.run.final-cut
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run util sound music title save-corruption
                          bootstrap-0 bootstrap-1
                          ram-display
                          ))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])
  (:import java.awt.image.BufferedImage)
  (:import java.io.File)
  (:import com.aurellem.gb.WaveWriter))


(def render-dir (File. user-home "proj/vba-clojure/render"))

(defn render-files!
  ([^File target-dir initial-state moves]
     (let [ram-map-dir (File. target-dir "ram-map")
           frames-dir (File. target-dir "frames")
           audio-file (File. target-dir "audio.wav")
           _ (.mkdirs ram-map-dir)
           _ (.mkdirs frames-dir)
           wave-writer (WaveWriter. audio-file)
           moves (vec moves)]
       (set-state! initial-state)
       (dorun
        (map
         (fn [index]
           (run-moves @current-state (vector (moves index)))

           ;; record audio
           (.process wave-writer (sound-bytes)
                     gb-sound-format)

           ;; write screenshot
           (write-png!
            @current-state
            (File. frames-dir (format "%07d.png" index)))

           ;; write ram-image
           (write-ram-image!
            @current-state
            (File. ram-map-dir (format "%07d.png" index))))
         (range (count moves))))
       (.cleanup wave-writer)))
  ([initial-state moves]
     (render-files! render-dir initial-state moves))
  ([moves]
     (render-files! (root) moves)))
                           

